@@ -30,7 +30,7 @@
  *     }
  *
  *     // Initialize the Google Drive client.
- *     let drive_client = GoogleDrive::new(token);
+ *     let drive_client = GoogleDrive::new(token.as_str().to_string());
  *
  *     // List drives.
  *     let drives = drive_client.list_drives().await.unwrap();
@@ -52,14 +52,13 @@ use std::time::Duration;
 use bytes::Bytes;
 use reqwest::{header, Client, Method, Request, StatusCode, Url};
 use serde::{Deserialize, Serialize};
-use yup_oauth2::AccessToken;
 
 /// The endpoint for the Google Drive API.
 const ENDPOINT: &str = "https://www.googleapis.com/drive/v3/";
 
 /// Entrypoint for interacting with the Google Drive API.
 pub struct GoogleDrive {
-    token: AccessToken,
+    token: String,
 
     client: Arc<Client>,
 }
@@ -68,7 +67,7 @@ impl GoogleDrive {
     /// Create a new Drive client struct. It takes a type that can convert into
     /// an &str (`String` or `Vec<u8>` for example). As long as the function is
     /// given a valid API Key and Secret your requests will work.
-    pub fn new(token: AccessToken) -> Self {
+    pub fn new(token: String) -> Self {
         let client = Client::builder().timeout(Duration::from_secs(360)).build();
         match client {
             Ok(c) => Self { token, client: Arc::new(c) },
@@ -77,7 +76,7 @@ impl GoogleDrive {
     }
 
     /// Get the currently set authorization token.
-    pub fn get_token(&self) -> &AccessToken {
+    pub fn get_token(&self) -> &str {
         &self.token
     }
 
@@ -96,12 +95,7 @@ impl GoogleDrive {
             Url::parse(&path).unwrap()
         };
 
-        // Check if the token is expired and panic.
-        if self.token.is_expired() {
-            panic!("token is expired");
-        }
-
-        let bt = format!("Bearer {}", self.token.as_str());
+        let bt = format!("Bearer {}", self.token);
         let bearer = header::HeaderValue::from_str(&bt).unwrap();
 
         // Set the default headers.
@@ -249,6 +243,46 @@ impl GoogleDrive {
         Ok(files_response.files)
     }
 
+    /// Get a folder by it's name.
+    pub async fn get_folder_by_name(&self, drive_id: &str, parent_id: &str, name: &str) -> Result<Vec<File>, APIError> {
+        let mut query = format!("name = '{}' and mimeType = 'application/vnd.google-apps.folder'", name);
+        if !parent_id.is_empty() {
+            query = format!("{} and '{}' in parents", query, parent_id);
+        }
+
+        // Build the request.
+        let request = self.request(
+            Method::GET,
+            "files".to_string(),
+            (),
+            Some(vec![
+                ("corpora", "drive".to_string()),
+                ("supportsAllDrives", "true".to_string()),
+                ("includeItemsFromAllDrives", "true".to_string()),
+                ("driveId", drive_id.to_string()),
+                ("q", query),
+            ]),
+            &[],
+            "",
+        );
+
+        let resp = self.client.execute(request).await.unwrap();
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                return Err(APIError {
+                    status_code: s,
+                    body: resp.text().await.unwrap(),
+                });
+            }
+        };
+
+        // Try to deserialize the response.
+        let files_response: FilesResponse = resp.json().await.unwrap();
+
+        Ok(files_response.files)
+    }
+
     /// List drives.
     pub async fn list_drives(&self) -> Result<Vec<Drive>, APIError> {
         // Build the request.
@@ -300,6 +334,13 @@ impl GoogleDrive {
             file.parents = vec![drive_id.to_string()];
         }
 
+        // Check if the folder exists.
+        let folders = self.get_folder_by_name(drive_id, parent_id, name).await.unwrap_or_default();
+        if !folders.is_empty() {
+            let f = folders.get(0).unwrap().clone();
+            return Ok(f.id);
+        }
+
         // Make the request and return the ID.
         let request = self.request(
             Method::POST,
